#ifndef DENSETRACK_H_
#define DENSETRACK_H_

#include <opencv/cv.h>
#include <opencv/highgui.h>
#include <ctype.h>
//#include <unistd.h>

#include <algorithm>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <fstream>
#include <iostream>
#include <vector>
#include <list>
#include <string>


#include "opencvDT/IplImageWrapper.h"
#include "opencvDT/IplImagePyramid.h"
#include "boost/numeric/ublas/vector.hpp"
#include "boost/numeric/ublas/io.hpp"


// parameters for descriptors
int patch_size = 32;
int nxy_cell = 2;
int nt_cell = 3;
bool fullOrientation = true;
float epsilon = 0.05;
const float min_flow = 0.4*0.4;
//const float PI = 3.14159;

// parameters for tracking
int start_frame = 0;
int end_frame = 1000000;
double quality = 0.001;
double min_distance = 5;
int init_gap = 1;
int track_length = 15;
char* fileVx, *fileVy, *outfile;

// parameters for the trajectory descriptor
const float min_var = sqrt(3.0);
const float max_var = 50;
const float max_dis = 20;

// parameters for multi-scale
int scale_num = 8;  
const float scale_stride = sqrt(2.0);

typedef struct TrackerInfo
{
    int trackLength; // length of the trajectory
    int initGap; // initial gap for feature detection
}TrackerInfo;

typedef struct DescInfo
{
    int nBins; // number of bins for vector quantization
    int fullOrientation; // 0: 180 degree; 1: 360 degree
    int norm; // 1: L1 normalization; 2: L2 normalization
    float threshold; //threshold for normalization
	int flagThre; // whether thresholding or not
    int nxCells; // number of cells in x direction
    int nyCells; 
    int ntCells;
	int dim; // dimension of the descriptor
    int blockHeight; // size of the block for computing the descriptor
    int blockWidth;
}DescInfo; 

typedef struct DescMat
{
    int height;
    int width;
    int nBins;
    float* desc;
}DescMat;

typedef struct STCDescMat
{
	int height;
	int width;
	int nBins;
	int nOffset;
	float* desc;
}STCDescMat;

class PointDesc
{
public:	
//     std::vector<float> hog;
//     std::vector<float> hof;
//     std::vector<float> mbhX;
//     std::vector<float> mbhY;
// 	std::vector<float> SHOG;	
// 	std::vector<float> THOG;
// 	std::vector<float> THOF;
// 	std::vector<float> TmbhX;
// 	std::vector<float> TmbhY;
	std::vector<float> CoMBHx_3D;
	std::vector<float> CoMBHy_3D;
	std::vector<float> CoHOF_3D;
	std::vector<float> CoHOG_3D;
// 	std::vector<float> SMBHx;
// 	std::vector<float> SMBHy;
// 	std::vector<float> SHOF;
    CvPoint2D32f point;
	// 	hog(hogInfo.nxCells * hogInfo.nyCells * hogInfo.nBins),
// 	SHOF(hofInfo.nxCells * hofInfo.nyCells * hofInfo.nBins * hofInfo.nBins * 2),
// 	SMBHx(mbhInfo.nxCells * mbhInfo.nyCells * mbhInfo.nBins* mbhInfo.nBins * 2),
// 		SMBHy(mbhInfo.nxCells * mbhInfo.nyCells * mbhInfo.nBins* mbhInfo.nBins * 2),

// 		hof(hofInfo.nxCells * hofInfo.nyCells * hofInfo.nBins),
// 		mbhX(mbhInfo.nxCells * mbhInfo.nyCells * mbhInfo.nBins),
// 		mbhY(mbhInfo.nxCells * mbhInfo.nyCells * mbhInfo.nBins),
// 		SHOG(hogInfo.nxCells * hogInfo.nyCells * hogInfo.nBins* 2),
// 		THOG(hogInfo.nxCells * hogInfo.nyCells * hogInfo.nBins* hogInfo.nBins*1),///////////////////
// 		THOF(hofInfo.nxCells * hofInfo.nyCells * hofInfo.nBins * hofInfo.nBins * 1),
// 		TmbhX(mbhInfo.nxCells * mbhInfo.nyCells * mbhInfo.nBins* mbhInfo.nBins * 1),
// 		TmbhY(mbhInfo.nxCells * mbhInfo.nyCells * mbhInfo.nBins* mbhInfo.nBins * 1),
    PointDesc(const DescInfo& hogInfo, const DescInfo& hofInfo, const DescInfo& mbhInfo, const CvPoint2D32f& point_)
		:CoHOG_3D((hogInfo.nxCells * hogInfo.nyCells+1 )* (hogInfo.nBins * hogInfo.nBins * 3 + hogInfo.nBins)),
		CoHOF_3D(( hofInfo.nxCells * hofInfo.nyCells + 1) * (hofInfo.nBins * hofInfo.nBins * 3) + hofInfo.nBins),
		CoMBHx_3D((mbhInfo.nxCells * mbhInfo.nyCells + 1) * (mbhInfo.nBins* mbhInfo.nBins * 3 + mbhInfo.nBins)),
		CoMBHy_3D((mbhInfo.nxCells * mbhInfo.nyCells + 1) * (mbhInfo.nBins* mbhInfo.nBins * 3 + mbhInfo.nBins)),
        point(point_)
    {}
};

class Track
{
public:
    std::list<PointDesc> pointDescs;
    int maxNPoints;

    Track(int maxNPoints_)
        : maxNPoints(maxNPoints_)
    {}

    void addPointDesc(const PointDesc& point)
    {
        pointDescs.push_back(point);
        if (pointDescs.size() > maxNPoints + 2) {
            pointDescs.pop_front();
		}
    }
};

#endif /*DENSETRACK_H_*/
